/***********************************************************************/
/*                                                                     */
/*                                                                     */
/* Program:                                                            */
/* --------                                                            */
/*                                                                     */
/* oscel.c                                                             */
/*                                                                     */
/*                                                                     */
/* Version:                                                            */
/* --------                                                            */
/*                                                                     */
/* First version: 2003-01-15                                           */
/* Latest update: 2003-02-18                                           */
/*                                                                     */
/*                                                                     */
/* Author of the program:                                              */
/* ----------------------                                              */
/*                                                                     */
/* Stefan Spaennare, Lund, Sweden                                      */
/*                                                                     */
/* E-mail: stefan@spaennare.se                                         */
/*                                                                     */
/* Web: http://www.spaennare.se/index.html                             */
/* Web: http://www.spaennare.se/ssnbody.html                           */
/*                                                                     */
/* If you have problems with the program please let me know!           */
/*                                                                     */
/*                                                                     */
/* Documentation:                                                      */
/* --------------                                                      */
/*                                                                     */
/* For a detailed description of the ssnbody nbody package see the     */
/* files ssnbody.pdf and ssnbody.ps on the ssnbody web page.           */
/*                                                                     */
/*                                                                     */
/* References:                                                         */
/* -----------                                                         */
/*                                                                     */
/* This program was written with permission from Staffan Soederhjelm,  */
/* Lund Observatory. The C-program was translated by the author from   */
/* a FORTRAN routine written by him.                                   */
/*                                                                     */
/***********************************************************************/

/*****************************************************************************/
/*                                                                           */
/* Notice:                                                                   */
/* =======                                                                   */
/*                                                                           */
/* I make no warranties that this program is (1) free of error, (2) con-     */
/* sistent with any standard merchantability, or (3) meeting the require-    */
/* ments of a particular application.  This software shall not, partly or    */
/* as a whole, participate in a process, whose outcome can result in injury  */
/* to a person or loss of property. It is solely designed for analytical     */
/* work.  Permission to use, copy, and distribute is hereby granted without  */
/* fee, providing that the header above including this notice appears in     */
/* all copies.                                                               */
/*                                                                           */
/*                                                         Stefan Spaennare  */
/*                                                                           */
/*****************************************************************************/


#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>


double pi=3.14159265358979323;


/* Converts radians to degrees. */

double deg(x)
double x;
{
   return(180.0*x/pi);
} /* deg */


/* Converts degrees to radians. */

double rad(x)
double x;
{
  return(pi*x/180.0);
} /* rad */


/*******************************************************************/
/*                                                                 */
/* Subroutine to calculate osculating elements from r,rdot vectors */
/* (for simplicity translated Montenbruck Pascal-routine)          */
/*                                                                 */
/* Input: gm           effective GM                                */
/*        elin(1-3)    x,y,z  (au)                                 */
/*        elin(4-6)    xdot,ydot,zdot  (au/day)                    */
/* Output: elout(1)    a   (au)                                    */
/*         elout(2)    e                                           */
/*         elout(3)    incl (rad)                                  */
/*         elout(4)    lom  (rad)  !from 9911 lom+stom             */
/*         elout(5)    stom (rad)                                  */
/*         elout(6)    M (mean anomaly, rad)                       */
/*                                                                 */
/*  SSo/960401, lom=long instead of arg,0-tpi constraint 9911      */
/*                                                                 */
/*******************************************************************/

void oscel(gm,elin,elout)
double gm;
double *elin;
double *elout;
{

   double hx,hy,hz,h;
   double u,r,v2,ax,c,s,e2,nu,ea,tpi;

   tpi=2.0*pi;

   /* Areal velocity => inc, stom */

   hx=elin[2]*elin[6]-elin[3]*elin[5];
   hy=elin[3]*elin[4]-elin[1]*elin[6];
   hz=elin[1]*elin[5]-elin[2]*elin[4];
   h=sqrt(hx*hx+hy*hy+hz*hz);

   elout[5]=atan2(hx,-hy);                    /* stom             */
   elout[3]=atan2(sqrt(hx*hx+hy*hy),hz);      /* incl             */
   u=atan2(elin[3]*h,-elin[1]*hy+elin[2]*hx); /* Arg. of latitude */

   /* Energy => a */

   r=sqrt(elin[1]*elin[1]+elin[2]*elin[2]+elin[3]*elin[3]);
   v2=elin[4]*elin[4]+elin[5]*elin[5]+elin[6]*elin[6];
   ax=1.0/(2.0/r-v2/gm);
   elout[1]=ax; /* a */

   /* ecc, E */

   c=1.0-r/ax; /* e*cos(E) */
   s=(elin[1]*elin[4]+elin[2]*elin[5]+elin[3]*elin[6])/sqrt(ax*gm); /* e*sin(E) */
   e2=c*c+s*s;

   elout[2]=sqrt(e2);             /* e                 */
   ea=atan2(s,c);                 /* Eccentric anomaly */
   elout[6]=ea-s;                 /* Mean anomaly      */
   nu=atan2(sqrt(1.0-e2)*s,c-e2); /* True anomaly      */
   elout[4]=u-nu;                 /* lom               */

   elout[4]=elout[4]+elout[5];

   if (elout[4] < 0.0) {
      elout[4]=elout[4]+tpi;
   } /* if */

   if (elout[4] > tpi) {
      elout[4]=elout[4]-tpi;
   } /* if */

   if (elout[6] < 0.0) {
      elout[6]=elout[6]+tpi;
   } /* if */

   if (elout[6] > tpi) {
      elout[6]=elout[6]-tpi;
   } /* if */

} /* oscel */


int main(argc,argv) 
int argc;
char *argv[];

{
   
   FILE *infile;

   int i;

   double x,y,z,vx,vy,vz,m;
   double x2,y2,z2,vx2,vy2,vz2,m2;
   double a,e,incl,lom,stom,man,mm;

   double elin[7];
   double elout[7];

   /* Infile contains position, velocity and mass for the Sun, planets, */
   /* the Moon (and optional some asteroids).                           */

   if (argc != 2) {
      printf("Usage: %s solsyst-verify-infile\n",argv[0]);
      exit(0);
   } /* if */
   

   if ((infile = fopen(argv[1],"r")) == NULL) {
      printf("File %s not found.\n",argv[1]);
      exit(0);
   } /* if */

   printf("\n");
   printf("nr      a               e");
   printf("               incl            lom");
   printf("             stom            M\n");

   /* Read data for the Sun (position, velocity and mass). */

   fscanf(infile,"%lf %lf %lf %lf %lf %lf %lf",&x2,&y2,&z2,&vx2,&vy2,&vz2,&m2);

   i=1;

   /* Read data for the planets and the Moon (position, velocity and mass). */

   while (fscanf(infile,"%lf %lf %lf %lf %lf %lf %lf",&x,&y,&z,&vx,&vy,&vz,&m) != EOF) {

      /* Calculate effective GM = G*(Msun+Mplanet) for body i. */

      mm=m+m2;

      /* Calculate relative coordinates for body i (i.e. subtract coordinates for the Sun). */

      elin[1]=x-x2;
      elin[2]=y-y2;
      elin[3]=z-z2;
      elin[4]=vx-vx2;
      elin[5]=vy-vy2;
      elin[6]=vz-vz2;

      /* Calculate osculating elements for body i. */

      oscel(mm,elin,elout);

      a=elout[1];
      e=elout[2];
      incl=deg(elout[3]);
      lom=deg(elout[4]);
      stom=deg(elout[5]);
      man=deg(elout[6]);

      /* Print osculating elements for body i. */

      printf("%2d %15.10lf %15.10lf %15.10lf %15.10lf %15.10lf %15.10lf\n"
	     ,i,a,e,incl,lom,stom,man);

      i++;

   } /* while */

   printf("\n");
 
} /* End */
